@@ -41,6 +41,7 @@ bundle = Table(
     Column('data_hash', String(63), nullable=True),
     Column('state', String(63), nullable=False),
     Column('owner_id', String(255), nullable=True),
+    Column('frozen', DateTime, nullable=True),  # When the bundle was frozen, if it is.
     Column('is_anonymous', Boolean, nullable=False, default=False),
     Column(
         'storage_type',
@@ -110,9 +111,7 @@ worksheet = Table(
     Column(
         'title', Unicode(255), nullable=True
     ),  # Short human-readable description of the worksheet
-    Column(
-        'frozen', DateTime, nullable=True
-    ),  # When the worksheet was frozen (forever immutable) if it is.
+    Column('frozen', DateTime, nullable=True),  # When the worksheet was frozen, if it is.
     Column('is_anonymous', Boolean, nullable=False, default=False),
     Column(
         'date_created', DateTime
