@@ -4,6 +4,7 @@ import mimetypes
 import os
 import re
 import sys
+import traceback
 import time
 from io import BytesIO
 from http.client import HTTPResponse
@@ -12,7 +13,7 @@ from bottle import abort, get, post, put, delete, local, request, response
 from codalab.bundles import get_bundle_subclass
 from codalab.bundles.uploaded_bundle import UploadedBundle
 from codalab.common import precondition, UsageError, NotFoundError
-from codalab.lib import canonicalize, spec_util, worksheet_util
+from codalab.lib import canonicalize, spec_util, worksheet_util, bundle_util
 from codalab.lib.server_util import (
     bottle_patch as patch,
     json_api_include,
@@ -333,6 +334,16 @@ def _update_bundles():
     bundle_uuids = [b.pop('uuid') for b in bundle_updates]
     check_bundles_have_all_permission(local.model, request.user, bundle_uuids)
     bundles = local.model.batch_get_bundles(uuid=bundle_uuids)
+    for bundle, update in zip(bundles, bundle_updates):
+        if "frozen" not in update:
+            bundle_util.check_bundle_not_frozen(bundle)
+        else:
+            # If we're freezing or unfreezing the bundle, check that
+            # the bundle is in a final state.
+            # If we're freezing, additionally check that the bundle is not already frozen.
+            bundle_util.check_bundle_freezable(bundle)
+            if update["frozen"]:
+                bundle_util.check_bundle_not_frozen(bundle)
 
     # Update bundles
     for bundle, update in zip(bundles, bundle_updates):
@@ -691,8 +702,17 @@ def _fetch_bundle_contents_blob(uuid, path=''):
     else:
         response.set_header('Content-Disposition', 'attachment; filename="%s"' % filename)
     response.set_header('Target-Type', target_info['type'])
-    response.set_header('X-Codalab-Target-Size', target_info['size'])
-
+    if target_info['type'] == 'file':
+        size = target_info['size']
+    elif not path and bundle_name:
+        # return data_size if the user requests the actual bundle
+        size = local.model.get_bundle_metadata([target.bundle_uuid], 'data_size').get(
+            target.bundle_uuid, 0
+        )
+    else:
+        # if request is for a subdir in a bundle then return 0
+        size = 0
+    response.set_header('X-Codalab-Target-Size', size)
     return fileobj
 
 
@@ -787,7 +807,11 @@ def _update_bundle_contents_blob(uuid):
             local.upload_manager.cleanup_existing_contents(bundle)
         msg = "Upload failed: %s" % err
         local.model.update_bundle(
-            bundle, {'state': State.FAILED, 'metadata': {'failure_message': msg}}
+            bundle,
+            {
+                'state': State.FAILED,
+                'metadata': {'failure_message': msg, 'error_traceback': traceback.format_exc()},
+            },
         )
         abort(http.client.BAD_REQUEST, msg)
 
@@ -806,7 +830,11 @@ def _update_bundle_contents_blob(uuid):
         # letting transient errors during upload fail the bundles prematurely.
         if finalize_on_failure:
             local.model.update_bundle(
-                bundle, {'state': State.FAILED, 'metadata': {'failure_message': msg}}
+                bundle,
+                {
+                    'state': State.FAILED,
+                    'metadata': {'failure_message': msg, 'error_traceback': traceback.format_exc()},
+                },
             )
 
         abort(http.client.INTERNAL_SERVER_ERROR, msg)
@@ -875,9 +903,10 @@ def delete_bundles(uuids, force, recursive, data_only, dry_run):
     check_bundles_have_all_permission(local.model, request.user, relevant_uuids)
 
     # Make sure we don't delete bundles which are active.
-    states = local.model.get_bundle_states(uuids)
+    bundles = local.model.batch_get_bundles(uuid=uuids)
+    states = [bundle.state for bundle in bundles]
     logger.debug('delete states: %s', states)
-    active_uuids = [uuid for (uuid, state) in states.items() if state in State.ACTIVE_STATES]
+    active_uuids = [uuid for (uuid, state) in zip(uuids, states) if state in State.ACTIVE_STATES]
     logger.debug('delete actives: %s', active_uuids)
     if len(active_uuids) > 0:
         raise UsageError(
@@ -888,6 +917,10 @@ def delete_bundles(uuids, force, recursive, data_only, dry_run):
             + 'can be deleted.'
         )
 
+    # Make sure we don't delete frozen bundles
+    for bundle in bundles:
+        bundle_util.check_bundle_not_frozen(bundle)
+
     # Make sure that bundles are not referenced in multiple places (otherwise, it's very dangerous)
     result = local.model.get_all_host_worksheet_uuids(relevant_uuids)
     for uuid, host_worksheet_uuids in result.items():
@@ -940,6 +973,8 @@ def set_bundle_permissions(new_permissions):
     )
     # Sequentially set bundle permissions
     for p in new_permissions:
+        bundle = local.model.get_bundle(p['object_uuid'])
+        bundle_util.check_bundle_not_frozen(bundle)
         local.model.set_group_bundle_permission(p['group_uuid'], p['object_uuid'], p['permission'])
 
 
