@@ -74,6 +74,7 @@ class BundleInfo(object):
         command,  # type: str
         data_hash,  # type: str
         state,  # type: State
+        frozen,  # type: Optional[str]
         is_anonymous,  # type: bool
         metadata,  # type: Dict[Any, Any]
         dependencies,  # type: List[Dict[str, str]]
@@ -86,6 +87,7 @@ class BundleInfo(object):
         self.command = command
         self.data_hash = data_hash
         self.state = state
+        self.frozen = frozen
         self.is_anonymous = is_anonymous
         self.metadata = metadata
         self.args = args
@@ -121,6 +123,7 @@ class BundleInfo(object):
             command=dct["command"],
             data_hash=dct["data_hash"],
             state=dct["state"],
+            frozen=dct.get("frozen"),
             is_anonymous=dct["is_anonymous"],
             metadata=dct["metadata"],
             dependencies=dct["dependencies"],
@@ -188,8 +191,6 @@ class BundleCheckinState(object):
         remote,  # type: str
         exitcode,  # type: Optional[str]
         failure_message,  # type: Optional[str]
-        cpu_usage,  # type: float
-        memory_limit,  # type: int
     ):
         self.uuid = uuid
         self.run_status = run_status
@@ -202,8 +203,6 @@ class BundleCheckinState(object):
         self.remote = remote
         self.exitcode = exitcode
         self.failure_message = failure_message
-        self.cpu_usage = cpu_usage
-        self.memory_limit = memory_limit
 
     @classmethod
     def from_dict(cls, dct):
@@ -219,8 +218,6 @@ class BundleCheckinState(object):
             remote=dct['remote'],
             exitcode=dct['exitcode'],
             failure_message=dct['failure_message'],
-            cpu_usage=dct['cpu_usage'],
-            memory_limit=dct['memory_limit'],
         )
 
     @property
